/***************************************************************************/
/* maszRDM - Zusatzprogramm fr MegaCAD(R) 2009, 2010, ...	
/*
/* wird maszRDM ohne Parameter aufgerufen, setzt, wechselt oder entfernt
/* es bei jedem klick auf ein Ma, dass Radius-, Durchmesser  oder Gewinde-
/* zeichen 'M'. 
/* Wird maszRDM mit dem Zeichen 'R', 'D' oder 'M' als Paramter aufgerufen, 
/* knnen mehrere Mae selektiert werden und es wird das jeweilige Zeichen
/* vor die Mae gesetzt. Ist das Zeichen schon vorhanden wird es entfernt.
/*
/* Aufruf Beispiele:
/* 1. InterP maszRDM.cip
/* 2. InterP maszRDM.cip R
/* 3. InterP maszRDM.cip D
/* 4. InterP maszRDM.cip M
/*
/* Version 1.0 (20.11.2010)
/* Version 1.01 (25.11.2010)
/* - ANSI/OEM Unterscheidung fr 'D'
/* - (char) Umwandlung fr 'D' gendert
/*
/* Copyright(C) 2010 JonCarpenter
/*
/* This program is free software; you can redistribute it and/or modify it
/* under the terms of the GNU General Public License as published by the 
/* Free Software Foundation; either version 3 of the License, or 
/* (at your option) any later version.
/* This program is distributed in the hope that it will be useful, but 
/* WITHOUT ANY WARRANTY; without even the implied warranty of 
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
/* General Public License for more details.
/* You should have received a copy of the GNU General Public License along 
/* with this program; if not, see <http://www.gnu.org/licenses/>
/*
/* 
/* MegaCAD(R) ist ein eingetragenes Warenzeichen der MEGATECH Software GmbH
/**************************************************************************/
#pragma HeapMin 100000
#pragma HeapMax 1000000

#include "std.h"      
#include "megatyp.h"
#include "megacad.h"

// dimstrmov verschiebt den String 'str' Zeichenweise ab dem Wert 'offset'
// um den Wert von 'move' vor oder zurck
void dimstrmov(char *str, int offset, int move)
{
	int ls = strlen(&str[offset]);				// Lnge Matext
	int lt1 = strlen(&str[offset+ls+1]);		// Lnge Toleranz 1
	int lt2 = strlen(&str[offset+ls+1+lt1+1]);	// Lnge Toleranz 2
	int i = 0;

	if(move != 0)
	{
		if(move > 0)
		{
			for(i = ls+1+lt1+1+lt2; i >= 0; i--)
			{
				str[offset+move+i] = str[offset+i];
			}
		}
		else
		{
			for(i=0; i < ls+1+lt1+1+lt2; i++)
			{
				str[offset+i] = str[offset-move+i];
			}
		}
	}
	return;
}

// setchar setzt das in 'c' bergebene Zeichen vor das entsprechende Ma
// ist das Zeichen schon vorhanden, wird es entfernt
void setchar(t_entity *ent, char c)
{	
	short pre = 0;

	// Prefix ermitteln
	pre = ent->data.dim.pref;

	switch(ent->data.dim.str[pre])
	{
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':   // wenn Zahl dann um eins verschieben c einfgen
				dimstrmov(ent->data.dim.str, pre, 1);
				ent->data.dim.str[pre] = c;
				break;
	default:	// wenn anderes Zeichen, dann 
				if(ent->data.dim.str[pre] == c)
				{
					// wenn c Zeichen, dann entferne es (um minus eins verschieben)
					dimstrmov(ent->data.dim.str, pre, -1); 
				}
				else
				{
					// tausche es aus
					ent->data.dim.str[pre] = c;
				}
				break;
	}
	// falls mit umgebenden Rechteck, dann Rechteckgre neu ermitteln
	if(ent->data.dim.mbox == 1)
	{
		GetDimVal(ent->type, 0, &ent->data.dim);
	}
	return;
}

// SellectLoop fr 'M'
short setR(void * arg)
{
	setchar((t_entity*)arg, 'R');

	ExChangeEntity((t_entity*)arg, NULL);

	return(CALL_SAVECOUNT);		//(CONTINUE_LOOP);
}

// SellectLoop fr 'D'
short setD(void * arg)
{
	t_entity * ent = (t_entity*) arg;
	
	if(ent->data.dim.fnt.CharSet == GT_ANSI)
	{
		setchar((t_entity*)arg, (char)-8);
	}
	else
	{
		setchar((t_entity*)arg, (char)-19);
	}

	ExChangeEntity((t_entity*)arg, NULL);

	return(CALL_SAVECOUNT);		
}

// SellectLoop fr 'M'
short setM(void * arg)
{
	setchar((t_entity*)arg, 'M');

	ExChangeEntity((t_entity*)arg, NULL);

	return(CALL_SAVECOUNT);		
}

// Hauptfunktion aufruf aus MegaCAD
extern "C" short MegaMain(char *file,  char *args) // extern "C" bei C-Interprter entfernen
{
	ulong		filter1, filter2;
	t_entity    ent;
	double		pnt2D[2];
	double		pnt3D[3];
	t_edta	*	edta = GetVarPtr();					//fr C-Interpreter in T_EDTA oder void ndern
	uint		edtaMax = GetMaxDataSize() - 1;
	short		pre = 0;
	ulong		oldID = 0;

	// Funtionstext und Maushilfe setzen
	SetCharSet(GT_ANSI);
    SetFuncText("Mavorsatz ndern");
    MouseHelp("Ma anklicken","zurck",HLP_INV(RED,WHITE,0));

	// Filter fr Auswahl der zulssigen Elemente (Mae)
	// (horizontale | vertikale	   | Winkel	      | Parallel/Fase | Radien     | Durchmesser  | Toleranzen)
	// (1<<E_H_DIM) | (1<<E_V_DIM) | (1<<E_A_DIM) | (1<<E_P_DIM)| (1<<E_R_DIM) | (1<<E_D_DIM) | (1<<E_T_DIM) 
    filter1 = ((1<<E_H_DIM) | (1<<E_V_DIM) | (1<<E_A_DIM) | (1<<E_P_DIM)); 
	filter2 = 0L;

	SelectInit(filter1, filter2);
	// erstes Zeichen von Parameter auswerten
	switch(args[0])
	{
	case 'R':	// Maauswahl mit 'R' versehen
				SelectLoop(setR, &ent, edta, NULL, edtaMax, TRUE);
				break;
	case 'D' :	// Maauswahl mit 'D' versehen
				SelectLoop(setD, &ent, edta, NULL, edtaMax, TRUE);
				break;
	case 'M':	// Maauswahl mit 'M' versehen
				SelectLoop(setM, &ent, edta, NULL, edtaMax, TRUE);
				break;
	default:	
		// Mae anklicken - Wechsel von ''->'R'->'D'->'M'->''
		while(ClickEntity(filter1,filter2,pnt2D,pnt3D,SNP_NIX,&ent,edta,edtaMax,NULL,FALSE))
		{
			// falls neues Element angeklickt altes fr Undo merken
			if(ent.id != oldID && oldID != 0) 
			{
				savecount();
				oldID = ent.id;
			}

			// Ma Prefix ermitteln
			pre = ent.data.dim.pref;

			switch(ent.data.dim.str[pre])
			{
			case '0':
			case '1':
			case '2':
			case '3':
			case '4':
			case '5':
			case '6':
			case '7':
			case '8':
			case '9':   // wenn Zahl, dann um eins verschieben 'R' einfgen
						dimstrmov(ent.data.dim.str, pre, 1);
						ent.data.dim.str[pre] = 'R';
						break;
			case 'R':	// wenn 'R' Zeichen, dann 'D' daraus machen
						// TT VF unterscheiden
						if(ent.data.dim.fnt.CharSet == GT_ANSI)
						{
							ent.data.dim.str[pre] = (char)-8; 
						}
						else
						{
							ent.data.dim.str[pre] = (char)-19;
						}
						break; 
			case -8 :	// wenn 'D' Zeichen, dann 'M' daraus machen
						if(ent.data.dim.fnt.CharSet == GT_ANSI)
						{
							ent.data.dim.str[pre] = 'M'; 
						}
						break; 
			case -19:	// wenn 'D' Zeichen, dann 'M' daraus machen
						if(ent.data.dim.fnt.CharSet != GT_ANSI)
						{
							ent.data.dim.str[pre] = 'M'; 
						}
						break;
			case 'M':	// wenn 'M' Zeichen, dann 'M' entfernen (um minus eins verschieben)
						dimstrmov(ent.data.dim.str, pre, -1); 
						break;
			default:	// wenn anderes Zeichen, dann mache vorerst nichts
						break;
			}
			// falls mit umgebenden Rechteck, dann Rechteckgre neu ermitteln
			if(ent.data.dim.mbox == 1)
			{
				GetDimVal(ent.type, 0, &ent.data.dim);
			}
			// Alte Elementdaten mit Neuen austauschen
			oldID = ExChangeEntity(&ent,NULL);
		}
		savecount();
	}
	return(0);
}
//end.	//fr C-Interpeter